<?php
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
require_once 'Zend/Search/Lucene/Analysis/Analyzer/Common.php';
require_once 'Zend/Search/Lucene/Analysis/TokenFilter/LowerCaseUtf8.php';

class TextAnalyzer extends Zend_Search_Lucene_Analysis_Analyzer_Common
{
	protected $isCJKAnalyzer;
	
	public function __construct($isCJKAnalyzer = false)
	{
		$this->isCJKAnalyzer = $isCJKAnalyzer;
		$this->addFilter(new Zend_Search_Lucene_Analysis_TokenFilter_LowerCaseUtf8());
	}
	
	public function reset()
	{
		$this->_position = 0;
		if (strcasecmp($this->_encoding, 'utf8' ) != 0  &&
			strcasecmp($this->_encoding, 'utf-8') != 0 ) {
				$this->_input = iconv($this->_encoding, 'UTF-8', $this->_input);
				$this->_encoding = 'UTF-8';
		}
	}
	
	public function nextToken()
	{
		$tokenStr = '';
		$startPos = $this->_position;
		$len = iconv_strlen($this->_input, 'UTF-8');
		if ($this->_position >= $len)
		{
			return null;
		}
		$firstChar = iconv_substr($this->_input, $this->_position, 1, 'UTF-8');
		$incorporation = $this->isIncorporation($firstChar);
		
		for ($i = $this->_position; $i < $len; $i++)
		{
			$char = iconv_substr($this->_input, $this->_position, 1, 'UTF-8');
			if ($incorporation xor $this->isIncorporation($char))
			{
				break;
			}
			else
			{
				if ($this->isCJKAnalyzer)
				{
					if ($incorporation && $this->_position!=0)
					{
						$lastchar = iconv_substr($this->_input, $this->_position-1, 1, 'UTF-8');
						if ($this->isIncorporation($lastchar))
						{
							$tokenStr = iconv_substr($this->_input, $this->_position++-1, 2, 'UTF-8');
							break;
						}
					}
					$tokenStr .= $char;
					$this->_position++;
				}
				else
				{
					$tokenStr .= $char;
					$this->_position++;
					if ($this->isIncorporation($char))
					{
						break;
					}
				}
			}
		}
		$token = $this->normalize(new Zend_Search_Lucene_Analysis_Token($tokenStr, $startPos, $this->_position-1));
		return $token;
	}
	
	public function isIncorporation($str)
	{
		if (preg_match('/^[\x{4e00}-\x{9fa5}\x{3130}-\x{318f}\x{ac00}-\x{d7a3}\x{0800}-\x{4e00}]+$/u', $str))
		{
			return true;
		} 
		return false;
	}
}
 ?>