<?php
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
require_once 'TextAnalyzer.php';
require_once 'Zend/Search/Lucene.php';
require_once 'Zend/Search/Lucene/Search/Query/Boolean.php';
require_once 'Zend/Search/Lucene/Search/Query/Phrase.php';
 
class TextParser
{
	private static $_instance = null;
	
	public $field = null;
	
	private static function _getInstance()
	{
		if (self::$_instance === null) {
            self::$_instance = new self();
        }
        return self::$_instance;
	}
	
	public static function parse($strQuery, $encoding)
    {	
		self::_getInstance();
		self::$_instance->field = Zend_Search_Lucene::getDefaultSearchField();
		$text = self::$_instance->_filterCharacter($strQuery);
		$clauses = self::$_instance->_getClauses($text);
		
		$query = new Zend_Search_Lucene_Search_Query_Boolean();
		foreach ($clauses as $clause) 
		{
			$subQuery = new Zend_Search_Lucene_Search_Query_Phrase();
			$tokens = Zend_Search_Lucene_Analysis_Analyzer::getDefault()->tokenize($clause, $encoding);
			$firstTerm = self::$_instance->_getTerm($tokens[0]->getTermText());
			$clauseField = $firstTerm["field"];
			$sign = $firstTerm["sign"];
	
			foreach ($tokens as $token)
			{
				$term = self::$_instance->_getTerm($token->getTermText());
				$subQuery->addTerm(new Zend_Search_Lucene_Index_Term($term["text"], $clauseField));
			}
			$query->addSubquery($subQuery, $sign);

			//$query->addSubquery($subQuery, null);
		}
		return $query;
	}
	
	public static function parseByGroup($queryGroup, $encoding)
	{
		$query = new Zend_Search_Lucene_Search_Query_Boolean();
		foreach ($queryGroup as $queryEntry) 
		{
			$subQuery = self::parse($queryEntry["queryString"] , $encoding);
			$query->addSubquery($subQuery, $queryEntry["sign"]);
		}
		return $query;
	}
	
	public function _filterCharacter($strQuery)
	{
		$newStr = str_replace('\'', '', $strQuery);
		$newStr = str_replace('"', ' ', $newStr);
		$newStr = str_replace(',', ' ', $newStr);
		return $newStr;
	}
	
	public function _getClauses($strQuery)
	{
		$i = 0;
		$parts = explode(" ", $strQuery);
		foreach ($parts as $e) 
		{
			if ($e != '') 
			{
				$clauses[$i++] = $e;
			}
		}
		return $clauses;
	}
	public function _getTerm($termText)
	{
		$signText = iconv_substr($termText, 0, 1, 'UTF-8');
		if ($signText == '+')
		{
			$sign = true;
			$termText = iconv_substr($termText, 1, iconv_strlen($termText, 'UTF-8')-1, 'UTF-8');
		}
		else if ($signText == '-')
		{
			$sign = false;
			$termText = iconv_substr($termText, 1, iconv_strlen($termText, 'UTF-8')-1, 'UTF-8');
		}
		else
		{
			$sign = null;
		}
		
		$pos = iconv_strpos($termText, ':', 0, 'UTF-8');
		if ($pos === false)
		{
			$result = array (
				"text"  => $termText,
				"field" => $this->field,
				"sign" => $sign
			);
		}
		else
		{
			$result = array (
				"text"  => iconv_substr($termText, $pos+1, iconv_strlen($termText, 'UTF-8')-$pos-1, 'UTF-8'),
				"field" => iconv_substr($termText, 0, $pos, 'UTF-8'),
				"sign" => $sign
			);
		}
		return $result;
	}
}
 ?>